@@ -14,6 +14,13 @@ from django.utils.http import urlencode
 # Internal Imports.
 from django_expanded_test_cases.constants import (
     ETC_DEBUG_PRINT,
+    ETC_USER_MODEL_IDENTIFIER,
+    ETC_DEFAULT_SUPER_USER_IDENTIFIER,
+    ETC_DEFAULT_ADMIN_USER_IDENTIFIER,
+    ETC_DEFAULT_STANDARD_USER_IDENTIFIER,
+    ETC_DEFAULT_INACTIVE_USER_IDENTIFIER,
+    ETC_DEFAULT_USER_PASSWORD,
+    ETC_GENERATE_USERS_WITH_REAL_NAMES,
     OUTPUT_ACTUALS_ERROR,
     OUTPUT_ACTUALS_MATCH,
     OUTPUT_ERROR,
@@ -78,31 +85,120 @@ class CoreTestCaseMixin:
         :param debug_print: Optional bool that indicates if debug output should print to console.
                             Param overrides setting value if both param and setting are set.
         """
-        # Generate "special case" test user instances.
-        # Guarantees that there will always be at least some default User models when tests are run.
-        cls.test_superuser = get_user_model().objects.create_user(
-            username='test_superuser',
-            password='password',
-            is_superuser=True,
-        )
-        cls.test_admin = get_user_model().objects.create_user(
-            username='test_admin',
-            password='password',
-            is_staff=True,
-        )
-        cls.test_user = get_user_model().objects.create_user(username='test_user', password='password')
-        cls.test_inactive_user = get_user_model().objects.create_user(
-            username='test_inactive',
-            password='password',
-            is_active=False,
-        )
-
         # Check user debug_print option.
         if debug_print is not None:
             cls._debug_print_bool = bool(debug_print)
         else:
             cls._debug_print_bool = ETC_DEBUG_PRINT
 
+    @classmethod
+    def set_up_test_data(cls, extra_usergen_kwargs=None):
+        """
+        Acts as the equivalent of the UnitTesting "setUpTestData()" function.
+
+        However, since this is not inheriting from a given TestCase, calling the literal function
+        here would override instead.
+
+        :param extra_usergen_kwargs: Optional extra kwargs to pass into the get_user_model().objects.create_user()
+                                     function.
+        """
+        if extra_usergen_kwargs is None:
+            extra_usergen_kwargs = {}
+        elif not isinstance(extra_usergen_kwargs, dict):
+            raise ValueError(
+                'The extra_usergen_kwargs value must be a dictionary of additional args used in the '
+                'get_user_model().objects.create_user() function.'
+            )
+
+        # Generate "special case" test user instances.
+        # Guarantees that there will always be at least some default User models when tests are run.
+
+        # Superuser model. Can access/see everything, regardless of permissions.
+        cls.test_superuser = get_user_model().objects.create_user(
+            **{ETC_USER_MODEL_IDENTIFIER: ETC_DEFAULT_SUPER_USER_IDENTIFIER},
+            password=ETC_DEFAULT_USER_PASSWORD,
+            is_superuser=True,
+            **extra_usergen_kwargs,
+        )
+        # Attempt to set fields and then save. If fields exist, then they will populate. Else are ignored.
+        if ETC_GENERATE_USERS_WITH_REAL_NAMES:
+            cls.test_superuser.first_name = 'John'
+            cls.test_superuser.last_name = 'Doe'
+            cls.test_superuser.name = 'John'
+        else:
+            cls.test_superuser.first_name = 'SuperUserFirst'
+            cls.test_superuser.last_name = 'SuperUserLast'
+            cls.test_superuser.name = 'SuperUserName'
+        if ETC_USER_MODEL_IDENTIFIER.lower() != 'email':
+            cls.test_superuser.email = 'super_user@example.com'
+        cls.test_superuser.save()
+        cls.test_superuser = cls.get_user(cls, ETC_DEFAULT_SUPER_USER_IDENTIFIER)
+
+        # Admin user model. Can access Django admin.
+        cls.test_admin = get_user_model().objects.create_user(
+            **{ETC_USER_MODEL_IDENTIFIER: ETC_DEFAULT_ADMIN_USER_IDENTIFIER},
+            password=ETC_DEFAULT_USER_PASSWORD,
+            is_staff=True,
+            **extra_usergen_kwargs,
+        )
+        # Attempt to set fields and then save. If fields exist, then they will populate. Else are ignored.
+        if ETC_GENERATE_USERS_WITH_REAL_NAMES:
+            cls.test_admin.first_name = 'Jenny'
+            cls.test_admin.last_name = 'Johnson'
+            cls.test_admin.name = 'Jenny'
+        else:
+            cls.test_admin.first_name = 'AdminUserFirst'
+            cls.test_admin.last_name = 'AdminUserLast'
+            cls.test_admin.name = 'AdminUserName'
+        if ETC_USER_MODEL_IDENTIFIER.lower() != 'email':
+            cls.test_admin.email = 'admin_user@example.com'
+        cls.test_admin.save()
+        cls.test_admin = cls.get_user(cls, ETC_DEFAULT_ADMIN_USER_IDENTIFIER)
+
+        # Inactive user model. Has "is_active" set to false, and cannot login.
+        cls.test_inactive_user = get_user_model().objects.create_user(
+            **{ETC_USER_MODEL_IDENTIFIER: ETC_DEFAULT_INACTIVE_USER_IDENTIFIER},
+            password=ETC_DEFAULT_USER_PASSWORD,
+            is_active=False,
+            **extra_usergen_kwargs,
+        )
+        # Attempt to set fields and then save. If fields exist, then they will populate. Else are ignored.
+        if ETC_GENERATE_USERS_WITH_REAL_NAMES:
+            cls.test_inactive_user.first_name = 'Clang'
+            cls.test_inactive_user.last_name = 'Zythor'
+            cls.test_inactive_user.name = 'Clang'
+        else:
+            cls.test_inactive_user.first_name = 'InactiveUserFirst'
+            cls.test_inactive_user.last_name = 'InactiveUserLast'
+            cls.test_inactive_user.name = 'InactiveUserName'
+        if ETC_USER_MODEL_IDENTIFIER.lower() != 'email':
+            cls.test_inactive_user.email = 'inactive_user@example.com'
+        cls.test_inactive_user.save()
+        cls.test_inactive_user = cls.get_user(cls, ETC_DEFAULT_INACTIVE_USER_IDENTIFIER)
+
+        # Standard user model.
+        cls.test_user = get_user_model().objects.create_user(
+            **{ETC_USER_MODEL_IDENTIFIER: ETC_DEFAULT_STANDARD_USER_IDENTIFIER},
+            password=ETC_DEFAULT_USER_PASSWORD,
+            **extra_usergen_kwargs,
+        )
+        # Attempt to set fields and then save. If fields exist, then they will populate. Else are ignored.
+        if ETC_GENERATE_USERS_WITH_REAL_NAMES:
+            cls.test_user.first_name = 'Sammy'
+            cls.test_user.last_name = 'Smith'
+            cls.test_user.name = 'Sammy'
+        else:
+            cls.test_user.first_name = 'UserFirst'
+            cls.test_user.last_name = 'UserLast'
+            cls.test_user.name = 'StandardUserName'
+        if ETC_USER_MODEL_IDENTIFIER.lower() != 'email':
+            cls.test_user.email = 'user@example.com'
+        cls.test_user.save()
+        cls.test_user = cls.get_user(cls, ETC_DEFAULT_STANDARD_USER_IDENTIFIER)
+
+        # Default user to run tests with is above "test_user".
+        cls.user = cls.test_user
+
     def sub_test(self):
         """
         Acts as the equivalent of the UnitTesting "subtTest()" function.
@@ -279,11 +375,13 @@ class CoreTestCaseMixin:
 
     # region User Management Functions
 
-    def get_user(self, user, password='password'):
+    def get_user(self, user, password=ETC_DEFAULT_USER_PASSWORD, extra_usergen_kwargs=None):
         """Returns user matching provided value.
 
-        :param user: User model, or corresponding username, to use.
+        :param user: User model, or corresponding user identifier, to use.
         :param password: Password str to assign to user.
+        :param extra_usergen_kwargs: Optional extra kwargs to pass into the get_user_model().objects.create_user()
+                                     function.
         :return: User object
         """
         # Check if instance is User model.
@@ -292,34 +390,50 @@ class CoreTestCaseMixin:
             pass
 
         # Handle all "special cases" for testing logic.
-        elif user == 'test_superuser':
+        elif user == ETC_DEFAULT_SUPER_USER_IDENTIFIER:
             user = self.test_superuser
-        elif user == 'test_admin':
+        elif user == ETC_DEFAULT_ADMIN_USER_IDENTIFIER:
             user = self.test_admin
-        elif user == 'test_user':
-            user = self.test_user
-        elif user == 'test_inactive':
+        elif user == ETC_DEFAULT_INACTIVE_USER_IDENTIFIER:
             user = self.test_inactive_user
-
+        elif user == ETC_DEFAULT_STANDARD_USER_IDENTIFIER:
+            user = self.test_user
         else:
             # Is not User model. Get or create.
+
+            # First validate optional extra kwargs.
+            if extra_usergen_kwargs is None:
+                extra_usergen_kwargs = {}
+            elif not isinstance(extra_usergen_kwargs, dict):
+                raise ValueError(
+                    'The extra_usergen_kwargs value must be a dictionary of additional args used in the '
+                    'get_user_model().objects.create_user() function.'
+                )
+
             try:
-                user = get_user_model().objects.get(username=str(user))
+                user = get_user_model().objects.get(
+                    **{ETC_USER_MODEL_IDENTIFIER: str(user)},
+                    **extra_usergen_kwargs,
+                )
             except get_user_model().DoesNotExist:
-                user = get_user_model().objects.create(username=str(user))
+                user = get_user_model().objects.create(
+                    **{ETC_USER_MODEL_IDENTIFIER: str(user)},
+                    **extra_usergen_kwargs,
+                )
 
         # Handle passwords.
-        password = str(password).strip()
-        if len(password) == 0:
-            # Empty password. Reset back to default.
-            password = 'password'
-        user.set_password(password)
-        user.unhashed_password = password
-        user.save()
+        if not hasattr(user, 'unhashed_password') or user.unhashed_password != password:
+            password = str(password).strip()
+            if len(password) == 0:
+                # Empty password. Reset back to default settings value.
+                password = ETC_DEFAULT_USER_PASSWORD
+            user.set_password(password)
+            user.unhashed_password = password
+            user.save()
 
         return user
 
-    def add_user_permission(self, user_permission, user='test_user'):
+    def add_user_permission(self, user_permission, user=ETC_DEFAULT_STANDARD_USER_IDENTIFIER):
         """Adds Permission to given user.
 
         :param user_permission: Permission to add.
@@ -353,7 +467,7 @@ class CoreTestCaseMixin:
         # Return user object in case user wants to run additional checks.
         return user
 
-    def add_user_group(self, user_group, user='test_user'):
+    def add_user_group(self, user_group, user=ETC_DEFAULT_STANDARD_USER_IDENTIFIER):
         """Adds Group to given user.
 
         :param user_group: Group to add.
