@@ -15,8 +15,11 @@ from . import CoreTestCaseMixin
 from django_expanded_test_cases.constants import (
     OUTPUT_ERROR,
     RESPONSE_DEBUG_CONTENT,
+    RESPONSE_DEBUG_HEADERS,
+    RESPONSE_DEBUG_CONTEXT,
     RESPONSE_DEBUG_MESSAGES,
     RESPONSE_DEBUG_SESSION,
+    RESPONSE_DEBUG_FORMS,
     RESPONSE_DEBUG_USER_INFO,
     OUTPUT_EMPHASIS,
 )
@@ -88,21 +91,60 @@ class ResponseTestCaseMixin(CoreTestCaseMixin):
         self._debug_print()
         self._debug_print(
             '{0} {1} {0}'.format('=' * 10, 'response.headers'),
-            fore=RESPONSE_DEBUG_CONTENT,
+            fore=RESPONSE_DEBUG_HEADERS,
             style=OUTPUT_EMPHASIS,
         )
 
         # Print out data, if present.
         if response_headers is not None and len(response_headers) > 0:
             for key, value in response_headers.items():
-                self._debug_print('    * "{0}": "{1}"'.format(key, value))
+                self._debug_print('    * "{0}": "{1}"'.format(key, value), fore=RESPONSE_DEBUG_HEADERS)
         else:
             self._debug_print('    No response headers found.')
         self._debug_print()
 
     def show_debug_context(self, response_context):
         """Prints debug response context data."""
-        raise NotImplementedError()
+
+        # Handle for potential param types.
+        if isinstance(response_context, HttpResponseBase):
+            response_context = response_context.context or {}
+
+        if response_context is not None:
+            # Attempt to access key object. If fails, attempt to generate dict of values.
+            try:
+                response_context.keys()
+            except AttributeError:
+                # Handling for:
+                #     * django.template.context.RequestContext
+                #     * django.template.context.Context
+                # No guarantee this will work for other aribtrary types.
+                # Handle as they come up.
+                temp_dict = {}
+                for context in response_context:
+                    temp_dict = {**temp_dict, **context}
+                response_context = temp_dict
+
+        self._debug_print()
+        self._debug_print(
+            '{0} {1} {0}'.format('=' * 10, 'response.context'),
+            fore=RESPONSE_DEBUG_CONTEXT,
+            style=OUTPUT_EMPHASIS,
+        )
+
+        # NOTE: Response context object is strange, in that it's basically a dictionary,
+        # and it allows .keys() but not .values(). Thus, iterate on keys only and pull respective value.
+        if response_context is not None and len(response_context.keys()) > 0:
+            # pass
+            for key in response_context.keys():
+                context_value = str(response_context.get(key))
+                # Truncate display if very long.
+                if len(context_value) > 80:
+                    context_value = '"{0}"..."{1}"'.format(context_value[:40], context_value[-40:])
+                self._debug_print('    * {0}: {1}'.format(key, context_value), fore=RESPONSE_DEBUG_CONTEXT)
+        else:
+            self._debug_print('    No context data found.', fore=RESPONSE_DEBUG_CONTEXT)
+        self._debug_print()
 
     def show_debug_session_data(self, client):
         """Prints debug response session data."""
@@ -123,11 +165,7 @@ class ResponseTestCaseMixin(CoreTestCaseMixin):
                 self._debug_print('    * {0}: {1}'.format(key, value), fore=RESPONSE_DEBUG_SESSION)
         else:
             self._debug_print('    No session data found.', fore=RESPONSE_DEBUG_SESSION)
-        self._debug_print('')
-
-    def show_debug_form_data(self, response_context):
-        """Prints debug response form data."""
-        raise NotImplementedError()
+        self._debug_print()
 
     def show_debug_messages(self, response_context):
         """Prints debug response message data."""
@@ -155,10 +193,81 @@ class ResponseTestCaseMixin(CoreTestCaseMixin):
             self._debug_print('    No context messages found.', fore=RESPONSE_DEBUG_MESSAGES)
         self._debug_print()
 
+    def show_debug_form_data(self, response_context):
+        """Prints debug response form data."""
+
+        # Handle for potential param types.
+        if isinstance(response_context, HttpResponseBase):
+            response_context = response_context.context
+
+        self._debug_print()
+        self._debug_print(
+            '{0} {1} {0}'.format('=' * 10, 'Form Data'),
+            fore=RESPONSE_DEBUG_FORMS,
+            style=OUTPUT_EMPHASIS,
+        )
+
+        # Check if form or formset data is actually present.
+        if 'form' in response_context or 'formset' in response_context:
+            # Form or formset present on page.
+
+            # Attempt to get form data.
+            form_present = False
+            if 'form' in response_context:
+                form_present = True
+                form = response_context['form']
+
+                # Print general form data.
+                self._debug_print('    Provided Form Fields:', fore=RESPONSE_DEBUG_FORMS)
+                fields_submitted = False
+                for key, value in form.data.items():
+                    self._debug_print('        {0}: {1}'.format(key, value), fore=RESPONSE_DEBUG_FORMS)
+                    fields_submitted = True
+                if not fields_submitted:
+                    self._debug_print('        No form field data submitted.', fore=RESPONSE_DEBUG_FORMS)
+
+                # Print form data errors if present.
+                if not form.is_valid():
+                    self._debug_print()
+                    if len(form.errors) > 0 or len(form.non_field_errors()) > 0:
+                        self._debug_print('    Form Invalid:'.format(not form.is_valid()), fore=RESPONSE_DEBUG_FORMS)
+                        if len(form.non_field_errors()) > 0:
+                            self._debug_print('        Non-field Frrors:', fore=RESPONSE_DEBUG_FORMS)
+                            for error in form.non_field_errors():
+                                self._debug_print('            {0}'.format(error), fore=RESPONSE_DEBUG_FORMS)
+
+                        if len(form.errors) > 0:
+                            self._debug_print('        Field Errors:', fore=RESPONSE_DEBUG_FORMS)
+                            for error in form.errors:
+                                self._debug_print('            {0}'.format(error), fore=RESPONSE_DEBUG_FORMS)
+
+                else:
+                    self._debug_print('    Form found and valid.', fore=RESPONSE_DEBUG_FORMS)
+
+            # Attempt to get formset data.
+            if 'formset' in response_context:
+                if form_present:
+                    self._debug_print()
+                formset = response_context['formset']
+
+                for form in formset:
+                    self._debug_print('Form(set) Errors:')
+                    for error in form.non_field_errors():
+                        self._debug_print('    {0}'.format(error), fore=RESPONSE_DEBUG_FORMS)
+                    for error in form.errors:
+                        self._debug_print('    {0}'.format(error), fore=RESPONSE_DEBUG_FORMS)
+        else:
+            # No identifiable form or formset data present on page.
+            self._debug_print('    No form data found.', fore=RESPONSE_DEBUG_FORMS)
+
+        self._debug_print()
+
     def show_debug_user_info(self, user):
         """Prints debug user data."""
 
-        self._debug_print()
+        # Imported here to prevent potential "Apps aren't loaded yet" error.
+        from django.contrib.auth.models import AnonymousUser
+
         self._debug_print(
             '{0} {1} {0}'.format('=' * 10, 'User Info'),
             fore=RESPONSE_DEBUG_USER_INFO,
@@ -189,7 +298,9 @@ class ResponseTestCaseMixin(CoreTestCaseMixin):
                 fore=RESPONSE_DEBUG_USER_INFO,
             )
 
-            self._debug_print()
+        elif isinstance(user, AnonymousUser):
+
+            self._debug_print('    Anonymous user. No user is logged in.', fore=RESPONSE_DEBUG_USER_INFO)
 
         else:
             self._debug_print('    * Invalid user "{0}" of type "{1}". Expected "{2}".'.format(
